<# 
.SYNOPSIS
		Run as a scheduled task, grants Lync policy to members of an Active Directory security group

.DESCRIPTION
		Run as a scheduled task, grants Lync policy to members of an Active Directory security group. The name of the group, and the type and name of the policy to be granted are all customizable.

.NOTES
    Version    	      	: v1.2 - See changelog at http://www.ehloworld.com/1346 
    Wish list						: 
    Rights Required			: local admin on server it's running on; CsUserAdministrator or CsAdministrator in Lync
    Sched Task Req'd		: Yes - install mode will automatically create scheduled task to run every 4 hours
    Lync Version				: N/A
    Lync Version				: 2010
    Author       				: Pat Richard, Exchange MVP
    Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
    Dedicated Post			: http://www.ehloworld.com/1348
    Disclaimer   				: You running this script means you won't blame me if this breaks your stuff.
    Info Stolen from 		: http://jeffwouters.nl/index.php/2012/06/powershell-function-to-get-all-nested-group-members-in-active-directory/

.LINK     
    http://www.ehloworld.com/1348

.INPUTS
		None. You cannot pipe objects to this script
		
.PARAMETER Install
		Create the scheduled task to run the script daily. It does NOT create the required Exchange receive connector.

.EXAMPLE 
		.\Grant-CsPolicyByADGroup.ps1
		
		Description
		-----------
		Looks at members of a defined AD group, and grants a Lync policy to each

.EXAMPLE 
		.\Grant-CsPolicyByADGroup.ps1 -install
		
		Description
		-----------
		Creates the scheduled task for the script to run everyday at 6am. It will prompt for the password for the currently logged on user.

#>
#Requires -Version 2.0
[CmdletBinding(SupportsShouldProcess = $True)]
param(
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, Mandatory = $false, HelpMessage = "Please supply group distinguished name")]
	[ValidateNotNullOrEmpty()]
	[string]$GroupDN,
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, Mandatory = $false, HelpMessage = "Please supply policy name")]
	[ValidateNotNullOrEmpty()]
	[string]$PolicyName,
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false, HelpMessage = "Please supply policy type")] 
	[ValidateNotNullOrEmpty()]
	[ValidateSet("Archiving","Client","ClientVersion","Conferencing","ExternalAccess","HostedVoicemail","Location","Mobility","Pin","Presence","Voice")]
	[string]$PolicyType = "ExternalAccess",
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false)] 
	[switch]$Install
)
[string]$ScriptName = $MyInvocation.MyCommand.Name
[string]$ScriptPathAndName = $MyInvocation.MyCommand.Definition

Write-Verbose "Defining functions"
function Set-ModuleStatus { 
	[cmdletBinding(SupportsShouldProcess = $true)]
	param	(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No module name specified!")] 
		[ValidateNotNullOrEmpty()]
		[string]$name
	)
	if(!(Get-Module -name "$name")) { 
		if(Get-Module -ListAvailable | ? {$_.name -eq "$name"}) { 
			Import-Module -Name "$name" 
			# module was imported
			return $true
		} else {
			# module was not available (Windows feature isn't installed)
			return $false
		}
	}else {
		# module was already imported
		return $true
	}
} # end function Set-ModuleStatus

function Remove-ScriptVariables {  
	[cmdletBinding(SupportsShouldProcess = $true)]
	param(
		[string]$path
	)
	$result = Get-Content $path |  
	ForEach { 
		if ( $_ -match '(\$.*?)\s*=') {      
			$matches[1]  | ? { $_ -notlike '*.*' -and $_ -notmatch 'result' -and $_ -notmatch 'env:'}  
		}  
	}  
	ForEach ($v in ($result | Sort-Object | Get-Unique)){		
		Remove-Variable ($v.replace("$","")) -EA 0
	}
} # end function Remove-ScriptVariables

function Install	{
	[cmdletBinding(SupportsShouldProcess = $true)]
	param()
	# http://technet.microsoft.com/en-us/library/cc725744(WS.10).aspx
	$error.clear()
	Write-Host "Creating scheduled task `"$ScriptName`"..."
	$TaskCreds = Get-Credential("$env:userdnsdomain\$env:username")
	$TaskPassword = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($TaskCreds.Password))	
	schtasks /create /tn $ScriptName /tr "$env:windir\system32\windowspowershell\v1.0\powershell.exe -command $ScriptPathAndName" /sc Hourly /MO 4 /ru $TaskCreds.UserName /rp $TaskPassword | Out-Null
	if (! $error){
		Write-Host "Installation complete!" -ForegroundColor green
	}else{
		Write-Host "Installation failed!" -ForegroundColor red
	}
	exit
} # end function Install

function Get-ADNestedGroupMembers{
	[cmdletbinding()]  
	param ( 
		[String] $Group
	) 
	Write-Verbose "Gathering group membership of $GroupDN"
	$Members = Get-ADGroupMember -Identity $Group
	$Members | % {	
		if ($_.ObjectClass -eq "group") {
			Write-Verbose "Getting nested group membership of $_.distinguishedName"
			Get-ADNestedGroupMembers -group $_.distinguishedName
		} else {
			$error.clear()
			[string]$name = $_.samaccountname
			Write-Verbose ""
			if ((Get-CsAdUser $name -ea 0).enabled){
				Write-Verbose "$name is enabled in Lync"
				[string]$cmd = "Grant-CS"+$PolicyType+"Policy -Identity "+$name+" -policyname `"$PolicyName`""
				Write-Verbose "Granting $PolicyType policy `"$PolicyName`""
				Invoke-Expression $cmd
				if (! $error){
					Write-Verbose "Successfully granted `"$PolicyName`" to $name"
				}else{
					Write-Verbose "Failed granting `"$PolicyName`" to $name: $error"
				}
			}else{
				Write-Verbose "$name is NOT enabled in Lync"
			}
			# return $_
		}
	}
}

if ($install){
	Write-Verbose "Install mode"
	Install
	Exit
}

$error.clear()
Write-Verbose "Loading modules"
Set-ModuleStatus Lync
Set-ModuleStatus ActiveDirectory

Get-ADNestedGroupMembers -group $GroupDN

Remove-ScriptVariables $ScriptPathAndName